# coding: utf-8
from django.db.models.base import ModelBase


def model_modifier_metaclass(meta_base=ModelBase, **params):
    u"""Возвращает метакласс, изменяющий параметры полей модели.

    :param dict params: Словарь с новыми значениями параметров полей. Ключ
        словаря должен содержать имя поля в модели (*field.attname*), а
        значение - словарь с новыми параметрами поля.

    .. note::

       Пример использования:

       .. code::

          class BaseModel(models.Model):
              name = models.CharField(max_length=150)

          modified_model_params = {
              'name': {
                  'max_length': 300
              }
          }
          class MyModel(BaseModel):
              # Модель с увеличенной до 300 символов длиной поля name.
              __metaclass__ = model_modifier_metaclass(**modified_model_params)

              class Meta:
                  verbose_name = u'Образец справочника'
    """
    class ModifiedModelBase(meta_base):
        def __new__(cls, name, bases, attrs):
            model = super(ModifiedModelBase, cls).__new__(
                cls, name, bases, attrs
            )

            for field_name, field_params in params.iteritems():
                field = model._meta.get_field(field_name)
                for param_name, param_value in field_params.iteritems():
                    assert hasattr(field, param_name), param_name
                    setattr(field, param_name, param_value)

            return model

    return ModifiedModelBase
